Module extendable

Module extendable 

Extendable parallel collection helpers.

Structs

ColAndPos
A collected payload together with its chunk positions.
IdxLen
A source index and the length of its chunk.
ThBegLen
Merge segment metadata for ordered thread values.

Traits

ParExtend
Extends an existing collection with items produced by a parallel iterator.
ParExtendCore
Parallel collection support for destinations that must be populated from multiple worker threads.