Module pools

Module pools 

Thread pool module

Macros

tasks
Macro helper to build a statically typed TaskQueue with the given tasks.

Structs

BasicPool
Native standard thread pool with persistent workers.
OncePool
A one-time-use thread pool.
Pool
Factory for creating thread pools with different characteristics.
Tasks
Entry point for building a statically typed TaskQueue to run in parallel via ThreadPool::run_all.

Traits

Scope
A scope within which work can be spawned onto a ThreadPool, bounded by the lifetimes of the borrowed environment.
TaskQueue
A strongly typed non-empty queue of heterogeneous elements.
ThreadPool
Abstraction for parallel execution environments and thread pool management.

Functions

max_num_threads_for_computation
Calculates the actual thread count for a computation considering multiple constraints.

Type Aliases

DefaultPool
Default thread pool