pub struct BasicPool { /* private fields */ }Expand description
Native standard thread pool with persistent workers.
This is the default thread pool when the std feature is enabled without
transient-pool, persistent-pool-rayon, or wasm features.
Note that the thread pool to be used for a parallel computation can be set by the
runner transformation separately for each parallel iterator.
Value of max_num_threads is determined as the minimum of:
- the available parallelism of the host obtained via
std::thread::available_parallelism(), and - the upper bound set by the environment variable “ORX_NUM_THREADS”, when set.
Implementations