Thread pool module
Macros
Structs
- Basic
Pool - Native standard thread pool with persistent workers.
- Once
Pool - A one-time-use thread pool.
- Pool
- Factory for creating thread pools with different characteristics.
- Tasks
- Entry point for building a statically typed
TaskQueueto run in parallel viaThreadPool::run_all.
Traits
- Scope
- A scope within which work can be spawned onto a
ThreadPool, bounded by the lifetimes of the borrowed environment. - Task
Queue - A strongly typed non-empty queue of heterogeneous elements.
- Thread
Pool - Abstraction for parallel execution environments and thread pool management.
Functions
- max_
num_ threads_ for_ computation - Calculates the actual thread count for a computation considering multiple constraints.
Type Aliases
- Default
Pool - Default thread pool